@@ -227,6 +227,7 @@ struct dhcp_state {
 	struct in_addr addr;
 	struct in_addr net;
 	struct in_addr dst;
+	uint8_t added;
 
 	char leasefile[sizeof(LEASEFILE) + IF_NAMESIZE];
 	time_t start_uptime;
@@ -271,9 +272,6 @@ ssize_t make_message(struct dhcp_message **, const struct interface *,
     uint8_t);
 int valid_dhcp_packet(unsigned char *);
 
-ssize_t write_lease(const struct interface *, const struct dhcp_message *);
-struct dhcp_message *read_lease(struct interface *);
-
 void dhcp_handleifa(int, struct interface *,
     const struct in_addr *, const struct in_addr *, const struct in_addr *);
 
